# SPDX-License-Identifier: MIT

# required for ctrl_rollback_clone and ctrl_reboot
. "$scripts_dir/rollback_clone.sh"
. "$scripts_dir/reboot.sh"
. "$scripts_dir/container_clear.sh"

ctrl_reset_default() {
	local dryrun=1

	while [ "$#" -gt 0 ]; do
		case "$1" in
		-f|--force)
			dryrun=0;
			shift;;
		*)
			reset_default_help
			exit 1
			;;
		esac
	done

	if [ "$dryrun" = 1 ]; then
		info "Run with dry-run mode."
	else
		lock_update
	fi

	# delete all network connections
	network_connections_delete

	# delete all dhcp settings
	dhcp_delete

	# delete all nat settings
	nat_delete

	# delete vpn container
	vpn_delete

	# delete ntp settings
	ntp_delete

	# delete user data
	user_data_delete

	if [ "$dryrun" = 1 ]; then
		info ""
		info "Re-run with --force to actually run the above commands"
	else
		if [ -n "$ERRORS" ]; then
			error "$ERRORS"
		fi
		# reflected on the B-side
		ctrl_rollback_clone
		# reboot
		ctrl_reboot
	fi
}

reset_default_help() {
	echo "Usage: abos-ctrl reset-default [reset-default options]"
	echo
	echo "  Delete all of the following settings/files:"
	echo "    Network settings(WWAN, WLAN, LAN, DHCP, NAT, VPN)"
	echo "    NTP settings"
	echo "    /var/app/volumes/*"
	echo "    /var/log/*"
	echo "  and reboot after the process is complete"
	echo
	echo "reset-default options commands:"
	echo "  -f/--force: actually delete the settings/files and reboot"
	echo "              If not set, no file is deleted (dry-run mode)."
}

ERRORS=""
soft_error() {
	warning "$@"
	ERRORS="$ERRORS
$*"
}

network_connections_delete() {
	# delete AP
	if [ -e "/etc/runlevels/default/hostapd" ]; then
		if [ "$dryrun" = 0 ]; then
			rc-service hostapd stop > /dev/null 2>&1 \
				|| soft_error "Could not stop hostapd"
			rc-update del hostapd > /dev/null \
				|| soft_error "Could not delete hostapd service"
			persist_file -d "/etc/runlevels/default/hostapd" \
				"/etc/hostapd/hostapd.conf" \
				|| soft_error "Could not delete hostapd setting files"
		fi
		info "rc-service hostapd stop"
		info "rc-update del hostapd"
		info "persist_file -d /etc/runlevels/default/hostapd /etc/hostapd/hostapd.conf"
	fi

	local ignore_if=wlan0
	if [ -e /sys/class/net/uap0 ] && [ -e /sys/class/net/mlan0 ]; then
		ignore_if=mlan0
	fi

	if [ -e /etc/NetworkManager/conf.d/90_abosweb_disable_wlan.conf ]; then
		if [ "$dryrun" = 0 ]; then
			persist_file -d /etc/NetworkManager/conf.d/90_abosweb_disable_wlan.conf \
				|| soft_error "Could not delete 90_abosweb_disable_wlan.conf"
		fi
		info "persist_file -d /etc/NetworkManager/conf.d/90_abosweb_disable_wlan.conf"
	fi
	if [ "$dryrun" = 0 ]; then
		if [ -e /sys/class/net/mlan0 ] || [ -e /sys/class/net/wlan0 ]; then
			nmcli d set "$ignore_if" managed yes
			nmcli d wifi list --rescan yes > /dev/null 2>&1
		fi
	fi

	if [ -e /etc/dnsmasq.d/dhcp_ap.conf ]; then
		if [ "$dryrun" = 0 ]; then
			persist_file -d "/etc/dnsmasq.d/dhcp_ap.conf" > /dev/null \
				|| soft_error "Could not delete dhcp_ap.conf"
			rc-service dnsmasq restart > /dev/null 2>&1 \
				|| soft_error "Could not restart dnsmasq"
		fi
		info "persist_file -d /etc/dnsmasq.d/dhcp_ap.conf"
		info "rc-service dnsmasq restart"
	fi

	# delete all connections
	if [ "$dryrun" = 0 ]; then
		rm -f /etc/NetworkManager/system-connections/* \
			|| soft_error "Could not remove system-connections"
		persist_file -r /etc/NetworkManager/system-connections > /dev/null \
			|| soft_error "Could not persist system-connections"
	fi
	info "rm -f /etc/NetworkManager/system-connections/*"
	info "persist_file -r /etc/NetworkManager/system-connections"
}

dhcp_delete() {
	for conf in /etc/dnsmasq.d/abos_web_dhcp_*.conf
	do
		[ -e "$conf" ] || break
		if [ "$dryrun" = 0 ]; then
			rm -f "$conf" || soft_error "Could not delete $conf"
		fi
		info "rm -f $conf"
	done
	if [ "$dryrun" = 0 ]; then
		persist_file -r /etc/dnsmasq.d \
			|| soft_error "Could not persist dnsmasq.d"
		rc-service dnsmasq restart > /dev/null 2>&1 \
			|| soft_error "Could not restart dnsmasq"
	fi
	info "persist_file -r /etc/dnsmasq.d"
	info "rc-service dnsmasq restart"
}

nat_delete() {
	# delete all nat settings
	local IFS=$'\n';
	for rule in $(iptables -t nat -S | grep abos-web-nat)
	do
		IFS=$' \t\n';
		if [ "$dryrun" = 0 ]; then
			# shellcheck disable=SC2086 ## split on purpose
			# note this won't work with multiple word arguments e.g.
			# -m comment --coment "some string", but abos-web never adds
			# such argument, and using eval would be worse.
			iptables -t nat ${rule/-A/-D}
		fi
		info "iptables -t nat ${rule/-A/-D}"
	done

	# delete all port forwarding settings
	IFS=$'\n';
	for rule in $(iptables -t nat -S | grep abos-web-pf)
	do
		IFS=$' \t\n';
		if [ "$dryrun" = 0 ]; then
			# shellcheck disable=SC2086 ## split on purpose
			iptables -t nat ${rule/-A/-D}
		fi
		info "iptables -t nat ${rule/-A/-D}"
	done

	IFS=$' \t\n';
	if [ "$dryrun" = 0 ]; then
		/etc/init.d/iptables save > /dev/null \
			|| soft_error "Could not save iptables"
		# prevent conflicts with container network settings
		sed -i -e '/NETAVARK/d' /etc/iptables/rules-save
		persist_file /etc/iptables/rules-save \
			|| soft_error "Could not persist rules-save"
	fi
	info "/etc/init.d/iptables save"
	info "sed -i -e '/NETAVARK/d' /etc/iptables/rules-save"
	info "persist_file /etc/iptables/rules-save"

	if [ -e "/etc/sysctl.d/abos_web_ipforward.conf" ]; then
		if [ "$dryrun" = 0 ]; then
			persist_file -d "/etc/sysctl.d/abos_web_ipforward.conf" \
				|| soft_error "Could not delete abos_web_ipforward.conf"
		fi
		info "persist_file -d /etc/sysctl.d/abos_web_ipforward.conf"
	fi
}

vpn_delete() {
	if podman ps -a | grep -q abos_web_openvpn; then
		if [ "$dryrun" = 0 ]; then
			podman stop abos_web_openvpn > /dev/null \
				|| soft_error "Could not stop abos_web_openvpn container"
			podman rm abos_web_openvpn > /dev/null \
				|| soft_error "Could not delete abos_web_openvpn container"
			persist_file -d /etc/atmark/containers/abos_web_openvpn.conf \
				|| soft_error "Could not delete abos_web_openvpn.conf"
			persist_file -r -d /etc/atmark/abos_web/openvpn \
				|| soft_error "Could not delete openvpn"
			persist_file -d /etc/dnsmasq.d/openvpn_dns.conf \
				|| soft_error "Could not delete openvpn_dns.conf"
			rc-service dnsmasq restart > /dev/null 2>&1 \
				|| soft_error "Could not restart dnsmasq"
		fi
		info "podman stop abos_web_openvpn"
		info "podman rm abos_web_openvpn"
		info "persist_file -d /etc/atmark/containers/abos_web_openvpn.conf"
		info "persist_file -r -d /etc/atmark/abos_web/openvpn"
		info "persist_file -d /etc/dnsmasq.d/openvpn_dns.conf"
		info "rc-service dnsmasq restart"
	fi
}

ntp_delete() {
	local modified_files=""
	for type in servers initstepslew
	do
		local name="$type.conf"
		local file="/etc/atmark/chrony.conf.d/$name"
		if [ -e "$file" ]; then
			if [ "$dryrun" = 0 ]; then
				rm -f "$file" || soft_error "Could not remove $file"
			fi
			info "rm -f $file"
			modified_files="$modified_files
$file"
		fi
		file="/etc/chrony/conf.d/$name"
		if [ -e "/lib/chrony.conf.d/$name" ] && [ -e "$file" ]; then
			if [ "$dryrun" = 0 ]; then
				rm -f "$file" || soft_error "Could not remove $file"
			fi
			info "rm -f $file"
			modified_files="$modified_files
$file"
		fi
	done

	local IFS=$'\n';
	[ -n "$modified_files" ] || return 0

	if [ "$dryrun" = 0 ]; then
		chronyd -p > /dev/null 2>&1
		rc-service chronyd restart > /dev/null \
			|| soft_error "Could not restart chronyd"
		# shellcheck disable=SC2086 ## split on purpose
		persist_file $modified_files > /dev/null \
			|| soft_error "Could not persist $modified_files"
	fi
	info "chronyd -p"
	info "rc-service chronyd restart"
	info "persist_file $modified_files"
}

user_data_delete() {
	get_appdev
	if [ "$dryrun" = 0 ]; then
		# stop all containers before remove files
		podman stop -a > /dev/null 2>&1
		find /var/log -mindepth 1 -delete

		# we do not want to delete external storage mounted
		# under these directories, for example "/var/app/volumes/usb".
		# but we want to delete btrfs subvol,
		# so we use btrfs_subvol_recursive_delete.
		basemount=$(mktemp -d /tmp/btrfs-root.XXXXXX) \
			|| error "Could not create tmpdir"
		mount -t btrfs "$appdev" "$basemount" \
			|| error "Could not mount appfs"
		btrfs_subvol_recursive_delete "/volumes"
		btrfs_subvol_create "volumes"
		# no need to umount/mount /var/app/volumes,
		# because it will reboot.
		umount "$basemount"
		rmdir "$basemount"
	fi
	info "podman stop -a"
	info "deleting /var/log/*"
	info "deleting /var/app/volumes/*"

	if [ -e "/etc/atmark/reset_default_list.txt" ]; then
		while read -r file; do
			[ -z "$file" ] && continue
			if [ -e "$file" ]; then
				if [ "$dryrun" = 0 ]; then
					persist_file -dr "$file" > /dev/null \
						|| soft_error "Could not delete $file"
				fi
				info "persist_file -dr $file"
			fi
		done < "/etc/atmark/reset_default_list.txt"
	fi
}
